@@ -4,6 +4,7 @@
 # ######################################### #
 
 import logging
+import warnings
 
 import numpy as np
 
@@ -48,14 +49,13 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
                       match_at_s=None,
                       particle_on_co=None,
                       R_matrix=None,
+                      W_matrix=None,
+                      method=None,
+                      nemitt_x=None, nemitt_y=None,
                       scale_with_transverse_norm_emitt=None,
                       weight=None,
                       particles_class=None,
-                      co_search_settings=None,
-                      steps_r_matrix=None,
-                      matrix_responsiveness_tol=None,
-                      matrix_stability_tol=None,
-                      symplectify=False,
+                      **kwargs, # They are passed to the twiss
                     ):
 
     """
@@ -113,9 +113,10 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         - R_matrix: 6x6 matrix defining the linearized one-turn map to be used
           for the transformation of the normalized coordinates into physical
           space.
-        - scale_with_transverse_norm_emitt: Tuple of two elements defining the
-          transverse normalized emittances used to rescale the provided
-          transverse normalized coordinates (x, px, y, py).
+        - W_matrix: 6x6 matrix with the eigenvalues of the one-turn map
+          (R_matrix). If provided, the R_matrix can be omitted.
+        - nemitt_x: transverse normalized emittance in x.
+        - nemitt_y: transverse normalized emittance in y.
         - weight: weights to be assigned to the particles.
         - at_element: location within the line at which particles are generated.
           It can be an index or an element name. It can be given  only if
@@ -134,6 +135,13 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     if particles_class is not None:
         raise NotImplementedError
 
+    # Deprecation warning for scale_with_transverse_norm_emitt
+    if scale_with_transverse_norm_emitt is not None:
+        warnings.warn(
+            "scale_with_transverse_norm_emitt is deprecated. Use "
+            "nemitt_x and nemitt_y instead.",
+            DeprecationWarning)
+
     if (particle_ref is not None and particle_on_co is not None):
         raise ValueError("`particle_ref` and `particle_on_co`"
                 " cannot be provided at the same time")
@@ -169,17 +177,6 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         logger.warning('Ignoring collective elements in particles generation.')
         tracker = tracker._supertracker
 
-    if tracker is not None:
-        if matrix_responsiveness_tol is None:
-            matrix_responsiveness_tol = tracker.matrix_responsiveness_tol
-        if matrix_stability_tol is None:
-            matrix_stability_tol = tracker.matrix_stability_tol
-
-    if matrix_responsiveness_tol is None:
-        matrix_responsiveness_tol=lnf.DEFAULT_MATRIX_RESPONSIVENESS_TOL
-    if matrix_stability_tol is None:
-        matrix_stability_tol=lnf.DEFAULT_MATRIX_STABILITY_TOL
-
     if zeta is None:
         zeta = 0
 
@@ -240,6 +237,9 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         assert mode == 'normalized_transverse'
         if isinstance(at_element, str):
             at_element = tracker.line.element_names.index(at_element)
+        assert R_matrix is None # Not clear if it is at the element or at start machine
+        if particle_on_co is not None:
+            assert particle_on_co._xobject.at_element == 0
 
     if match_at_s is not None:
         import xtrack as xt
@@ -258,7 +258,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             "specified `at_element`. No active element can be present in between."
             )
         (tracker_rmat, _
-            ) = xt.twiss_from_tracker._build_auxiliary_tracker_with_extra_markers(
+            ) = xt.twiss._build_auxiliary_tracker_with_extra_markers(
                 tracker=tracker, at_s=[match_at_s], marker_prefix='xpart_rmat_')
         at_element_tracker_rmat = tracker_rmat.line.element_names.index(
                                                                  'xpart_rmat_0')
@@ -267,34 +267,22 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         at_element_tracker_rmat = at_element
 
     if mode == 'normalized_transverse':
-        if particle_on_co is None:
-            assert tracker is not None
-            particle_on_co = tracker.find_closed_orbit(
-                particle_co_guess=Particles(
-                    x=0, px=0, y=0, py=0, zeta=0, delta=0.,
-                    **ref_dict),
-                co_search_settings=co_search_settings)
+
+        if W_matrix is None:
+            if method is not None:
+                kwargs['method'] = method
+            tw = tracker_rmat.twiss(particle_on_co=particle_on_co,
+                                    particle_ref=particle_ref,
+                                    R_matrix=R_matrix, **kwargs)
+            tw_state = tw.get_twiss_init(at_element=
+                (at_element_tracker_rmat if at_element_tracker_rmat is not None else 0))
+
+            WW = tw_state.W_matrix
+            particle_on_co = tw_state.particle_on_co
         else:
-            assert particle_on_co._capacity == 1
+            WW = W_matrix
 
-        if not isinstance(particle_on_co._buffer.context, xo.ContextCpu):
-            particle_on_co = particle_on_co.copy(_context=xo.ContextCpu())
-
-        assert particle_on_co.at_element[0] == 0
-        assert particle_on_co.s[0] == 0
-        assert particle_on_co.state[0] == 1
-
-        if at_element_tracker_rmat is not None:
-            # Match in a different position of the line
-            assert at_element_tracker_rmat > 0
-            part_co_ctx = particle_on_co.copy(_context=tracker_rmat._buffer.context)
-            tracker_rmat.track(part_co_ctx, num_elements=at_element_tracker_rmat)
-            particle_on_co = part_co_ctx.copy(_context=xo.ContextCpu())
-
-        if R_matrix is None:
-            # R matrix at location defined by particle_on_co.at_element
-            R_matrix = tracker_rmat.compute_one_turn_matrix_finite_differences(
-                particle_on_co=particle_on_co, steps_r_matrix=steps_r_matrix)
+        WWinv = np.linalg.inv(WW)
 
         num_particles = _check_lengths(num_particles=num_particles,
             zeta=zeta, delta=delta, x_norm=x_norm, px_norm=px_norm,
@@ -302,32 +290,30 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
 
         if scale_with_transverse_norm_emitt is not None:
             assert len(scale_with_transverse_norm_emitt) == 2
+            assert nemitt_x is None and nemitt_y is None, (
+                "If `scale_with_transverse_norm_emitt` is provided, "
+                "`nemitt_x` and `nemitt_y` should not be provided.")
 
             nemitt_x = scale_with_transverse_norm_emitt[0]
             nemitt_y = scale_with_transverse_norm_emitt[1]
 
-            gemitt_x = nemitt_x/particle_ref.beta0/particle_ref.gamma0
-            gemitt_y = nemitt_y/particle_ref.beta0/particle_ref.gamma0
+        if nemitt_x is None:
+            nemitt_x = 1
+        if nemitt_y is None:
+            nemitt_y = 1
 
-            x_norm_scaled = np.sqrt(gemitt_x) * x_norm
-            px_norm_scaled = np.sqrt(gemitt_x) * px_norm
-            y_norm_scaled = np.sqrt(gemitt_y) * y_norm
-            py_norm_scaled = np.sqrt(gemitt_y) * py_norm
-        else:
-            x_norm_scaled = x_norm
-            px_norm_scaled = px_norm
-            y_norm_scaled = y_norm
-            py_norm_scaled = py_norm
+        gemitt_x = nemitt_x/particle_ref.beta0/particle_ref.gamma0
+        gemitt_y = nemitt_y/particle_ref.beta0/particle_ref.gamma0
 
-        WW, WWinv, Rot = lnf.compute_linear_normal_form(R_matrix,
-                symplectify=symplectify,
-                responsiveness_tol=matrix_responsiveness_tol,
-                stability_tol=matrix_stability_tol)
+        x_norm_scaled = np.sqrt(gemitt_x) * x_norm
+        px_norm_scaled = np.sqrt(gemitt_x) * px_norm
+        y_norm_scaled = np.sqrt(gemitt_y) * y_norm
+        py_norm_scaled = np.sqrt(gemitt_y) * py_norm
 
         # Transform long. coordinates to normalized space
         XX_long = np.zeros(shape=(6, num_particles), dtype=np.float64)
-        XX_long[4, :] = zeta - particle_on_co.zeta
-        XX_long[5, :] = pzeta - particle_on_co.ptau / beta0
+        XX_long[4, :] = zeta - particle_on_co._xobject.zeta[0]
+        XX_long[5, :] = pzeta - particle_on_co._xobject.ptau[0] / beta0
 
         XX_norm_scaled = np.dot(WWinv, XX_long)
 
@@ -339,12 +325,12 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         # Transform to physical coordinates
         XX = np.dot(WW, XX_norm_scaled)
 
-        XX[0, :] += particle_on_co.x
-        XX[1, :] += particle_on_co.px
-        XX[2, :] += particle_on_co.y
-        XX[3, :] += particle_on_co.py
-        XX[4, :] += particle_on_co.zeta
-        XX[5, :] += particle_on_co.ptau / beta0
+        XX[0, :] += particle_on_co._xobject.x[0]
+        XX[1, :] += particle_on_co._xobject.px[0]
+        XX[2, :] += particle_on_co._xobject.y[0]
+        XX[3, :] += particle_on_co._xobject.py[0]
+        XX[4, :] += particle_on_co._xobject.zeta[0]
+        XX[5, :] += particle_on_co._xobject.ptau[0] / beta0
 
     elif mode == 'set':
 
@@ -412,11 +398,12 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
 
     if at_element is not None:
         if match_at_s is not None:
-            particles.s[:num_particles] = particle_on_co.s[0] + length_aux_drift
+            particles.s[:num_particles] = particle_on_co._xobject.s[0] + length_aux_drift
         else:
             assert particle_on_co.at_element[0] == at_element
-            particles.s[:num_particles] = particle_on_co.s[0]
+            particles.s[:num_particles] = particle_on_co._xobject.s[0]
         particles.at_element[:num_particles] = at_element
+
         particles.start_tracking_at_element = at_element
 
     return particles
